package bg.bas.dcl.applications;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.general.FileHandler;
import bg.bas.dcl.monolingual.bg.TextProcessor;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerb;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerbsVerifier;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbFormFormalism;
import bg.bas.dcl.multilingual.ClauseSplitting.ClauseSplittingFileProcessor;
import bg.bas.dcl.multilingual.ClauseSplitting.RuleBasedClauseSplitter;

public class LongSentencesExtractor {
	
	ArrayList<String> authors = new ArrayList<String>();
	RuleBasedClauseSplitter rbcs ;
	ClauseSplittingFileProcessor csfp = new ClauseSplittingFileProcessor();
	TextProcessor tp = new TextProcessor();
	VerbFormFormalism vff = new VerbFormFormalism();		
	ClauseVerbsVerifier cvv;

	public static void main (String[] args){
		
		String modelsFile = "/home/ivelina/workspace/ZCORPUSTOOLSTEST/" +
				"ClauseSplitting/resources/verb-form-models.txt";
		String conjFile = "/home/ivelina/workspace/ZCORPUSTOOLSTEST/" +
				"ClauseSplitting/resources/conj.txt";
		String specadvFile = "/home/ivelina/workspace/ZCORPUSTOOLSTEST/" +
				"ClauseSplitting/resources/specadv.txt";		
		String verbsFile = "/home/ivelina/workspace/ZCORPUSTOOLSTEST/" +
				"ClauseSplitting/resources/special-verbs.txt";  
		String punctExcFile = "/home/ivelina/workspace/ZCORPUSTOOLSTEST/" +
				"ClauseSplitting/resources/strong-punct-exceptions.txt";
		

		LongSentencesExtractor lse = new LongSentencesExtractor();

		lse.vff.readModelsFromFile(modelsFile);
		lse.cvv = new ClauseVerbsVerifier(verbsFile);
			
		lse.authors.add("Иван Вазов");
		lse.authors.add("Елин Пелин");
		lse.authors.add("Йордан Йовков");
		lse.authors.add("Емилиан Станев");
		lse.authors.add("Димитър Талев");
		lse.authors.add("Димитър Димов");
		
		String corpusdir = "/home/ivelina/SVN_CORPUS/BulNC/D-Fiction/";
		String metadir   = "/home/ivelina/SVN_CORPUS/BulNC-METADATA/D-Fiction/";
		
		String outfile   = "/home/ivelina/WORK-DCL/TASKS/LIT-SYNTAX/longsentences-results.txt";
		
		
		lse.extractLongSentences(corpusdir, metadir, outfile, 10);
		
	}
	
	int exctr = 0;
	public void extractLongSentences(String corpusdir, String metadir, String outfile, int minnumberofclauses){
		
		FileHandler fh = new FileHandler();

		try{
			Writer out = new OutputStreamWriter(
					new FileOutputStream(outfile));
			Writer outhtml = new OutputStreamWriter(
					new FileOutputStream(outfile+".html"));
			
			TextProcessor tp = new TextProcessor();

			outhtml.write("<head><meta charset=\"UTF-8\"></head>");
			
			List<File> list = fh.getFileListing(new File(corpusdir));
			
			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (f.getName().startsWith(".")) continue;
				if (f.getName().contains("~")) continue;
				
				System.out.println(f.getAbsolutePath());
				
				String meta = f.getAbsolutePath()
						.replace(corpusdir, metadir)
						.replace(".txt", ".json");
				File fmeta = new File(meta);
				if (!fmeta.exists()) continue;
				
				JSONObject record = readJSON (fmeta);
				
				System.out.println(((JSONObject) record.get("bg")).toJSONString());
				
				JSONObject bgobj = (JSONObject) record.get("bg");
				if (authors.contains((String)bgobj.get("автор"))){
					
					Scanner s = new Scanner(f, "UTF-8");
					
					while (s.hasNextLine()){
						String line = s.nextLine();
						
						line = line
								.replaceAll("\\!…", "\\!")
								.replaceAll("\\?…", "\\?")
								.replaceAll(": –", "\\. ")
								.replaceAll("…", "\\. ")
								.replaceAll(" Тютюн [0-9]+ ", " ")
								.replaceAll(" Осъдени души [0-9]+ ", " ")
								.replaceAll(" - ", " – ")
								.replaceAll(": «", "\\. „")
								.replaceAll("«", "„")
								.replaceAll("»", "“")
								;
						
						String[] sents = tp.splitToSentences(line);
						
						int ctr = 0;
						while (ctr < sents.length) {
							
							String sentt = sents[ctr];
							
							ctr++;
							
							if (sentt.length() < 500) {
								
								continue;
							}
							
							Sentence sent1 = readSentenceWithTagging(sentt);
							Sentence sent = new Sentence();
							
							for (int j = 0; j< sent1.size(); j++) {
								Word w = sent1.get(j);
								sent.add(w);
								
								if (w.tokenTag.endsWith("<S>")) {
									
									processSentence(bgobj, sent, out, outhtml);
									
									sent = new Sentence();
									
								}
							}
							
						
						}
						
					}
					
					s.close();
					System.out.println("EXAMPLES SO FAR: "+ exctr);
					
				}

//				System.exit(0);
			}

			out.flush();
			out.close();
			
			outhtml.flush();
			outhtml.close();
			
		} catch(Exception e){
			e.printStackTrace();
		}

		
	}
	
	ArrayList<String> pastExamples = new ArrayList<String>();
	
	public void processSentence(JSONObject bgobj, Sentence sent, Writer out, Writer outhtml){
		
		ArrayList<VerbForm> verbs1 = vff.findForms(sent);
		ArrayList<ClauseVerb> verbs = new ArrayList<ClauseVerb>();
		
		verbs.addAll(cvv.reviseVerbs(sent, verbs1));
		
		
		
		if (verbs.size() > 9){
			
			String info = 
					   bgobj.get("път в корпуса")+ "\n"
					+  bgobj.get("автор")+ ". "
					+ bgobj.get("заглавие на текста")+ ". "
					+ (!bgobj.get("дата на публикуване").equals("")?
							(bgobj.get("дата на публикуване")+ ". ") : "")											
					+ (!bgobj.get("издателство").equals("")?
							(bgobj.get("издателство")+ ". ") : "")											
					+ (!bgobj.get("място на издаване").equals("")?
							(bgobj.get("място на издаване")+ ". ") : "")											
								;
			
			try {
				String text = sent.getPlainText()
						.replaceAll(" \\.", "\\.")
						.replaceAll(" ,", ",")
						.replaceAll(" :", ":")
						.replaceAll(" ;", ";")
						.replaceAll(" \",", "\",")
						.replaceAll(" \"\\.", "\"\\.")
						.replaceAll(" \\?", "\\?")
						.replaceAll(" \\!", "\\!")
						.replaceAll(" \\)", "\\)")
						.replaceAll("\\( ", "\\(")
						.replaceAll(" “", "“")
						.replaceAll("„ ", "„")
						;
				
				if (!pastExamples.contains(text)){
					out.write(text + "\n\n");
					out.write(info + "\n\n");
					out.write("* * * * * * * * *\n\n");
					out.flush();
	
					outhtml.write(text + "<br><br>\n");
					outhtml.write(info.replace("\n", "<br>") + "<br><br>\n");
					outhtml.write("* * * * * * * * *<br><br>\n");
					outhtml.flush();
				
					exctr++;
					
					pastExamples.add(text);
					
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}


	}
	
	public JSONObject readJSON (File f){
		
		JSONParser parser = new JSONParser();

		try {

			InputStreamReader isr = new InputStreamReader(
					new FileInputStream(f), "UTF-8");
			JSONObject jobj = (JSONObject)parser.parse(isr);
			
			isr.close();
			
			return jobj;
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	
	public Sentence readSentenceWithTagging(String l){
		
		Sentence sent = new Sentence();
		String[] result = TextProcessor.applyServiceToText(l, 2).split("\n");

		for (String s : result) {
			
			Word t = new Word();
			t.readFromVertical(s);
			
			sent.add(t);
		}
		
		return sent;
	}

}
