package bg.bas.dcl.general;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

public class InitialParametersReader {
	
	public static final String INPUT_FOLDER = "input-folder";
	public static final String ENTITY = "entity";
	public static final String ENTITY_TAG = "entity-tag";
	public static final String LEMMA_FORMS = "lemma-forms";
	public static final String FILE_FORMAT = "file-format";
	public static final String INPUT_NAME = "input-name";
	public static final String OUTPUT_FOLDER = "output-folder";
	public static final String OUTPUT_ORDER = "output-order";
	public static final String MAX_DICT_SIZE = "max-dict-size";
	
	public static HashMap<String,ArrayList<String>> readInitParameters(String filename){
			
		HashMap<String,ArrayList<String>> parameters = new HashMap<String,ArrayList<String>>();
		
		try{
			
			Scanner s = new Scanner(new File(filename));
			
			while (s.hasNextLine()) {
				String l = s.nextLine().trim();
				
				if (l.length() == 0) continue; // ignore empty lines
				if (l.startsWith("#")) continue; // ignore commented lines
				if (l.startsWith("//")) continue; // ignore commented lines
				
				String[] data = l.split("=");
				
				if (data.length < 2) continue;
				
				String key = data[0].trim();
				ArrayList<String> value = new ArrayList<String>();
				
				if (parameters.containsKey(key)) value = parameters.get(key);
				
				value.add(data[1].trim());
				
				parameters.put(key, value);
				
			}
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
		return parameters;
	}

	// columns keycol; valuecol
	public static HashMap<String,String> readDictionaryFromFile(
			String dictfile,
			int keycol, int valuecol){
		HashMap<String,String> dict = new HashMap<String,String>();
		
		try{
			
			Scanner s = new Scanner(new File(dictfile),"UTF-8");
			
			while(s.hasNextLine()) {
				
				String  l = s.nextLine();//.trim();
				
//				System.out.println(l);
				if (l.equals("")) continue;
				
				String[] data = l.split("\t");
				
				if (valuecol == -1) {
					dict.put(data[keycol], l);
					continue;
				}
				if (data.length<Math.max(keycol,valuecol)+1) continue;
				
				dict.put(data[keycol], data[valuecol]);
				
			}
			
			s.close();
		} catch(Exception e){
			e.printStackTrace();
			
		}
		
		
		return dict;
	}

	public static ArrayList<String> readListFromFile(String file){
		ArrayList<String> dict = new ArrayList<String>();
		
		try{
			
			Scanner s = new Scanner(new File(file),"UTF-8");
			
			while(s.hasNextLine()) {
				
				String  l = s.nextLine().trim();
				if (l.equals("")) continue;
				
						
				if (!dict.contains(l)) dict.add(l);
				
			}
			
			s.close();
		} catch(Exception e){
			e.printStackTrace();
			
		}
		
		
		return dict;
	}

}
