package bg.bas.dcl.monolingual.en;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;
import java.util.Scanner;

import edu.stanford.nlp.ling.CoreAnnotations.LemmaAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.NamedEntityTagAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.PartOfSpeechAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.SentencesAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TextAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TokensAnnotation;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.trees.TreeCoreAnnotations.TreeAnnotation;
import edu.stanford.nlp.trees.semgraph.SemanticGraph;
import edu.stanford.nlp.trees.semgraph.SemanticGraphCoreAnnotations.CollapsedCCProcessedDependenciesAnnotation;
import edu.stanford.nlp.util.CoreMap;

import opennlp.tools.postag.POSModel;
import opennlp.tools.postag.POSTaggerME;
import opennlp.tools.sentdetect.SentenceDetectorME;
import opennlp.tools.sentdetect.SentenceModel;
import opennlp.tools.tokenize.Tokenizer;
import opennlp.tools.tokenize.TokenizerME;
import opennlp.tools.tokenize.TokenizerModel;

import bg.bas.dcl.general.FileHandler;

public class TextProcessorEn {
	
	long sent_ctr = 1;
	String resourceDir = "";
	
	HashMap<String,String> tagset = new HashMap<String,String>();
	String defaultTag = "M";

	public final String ENCODING = "UTF-8";
	public final String NL = System.getProperty("line.separator");

	SentenceDetectorME sentenceDetector = null;
	Tokenizer tokenizer = null;
	POSTaggerME tagger = null;

	TokenTaggerEn tokenTagger = null;
	
	public TextProcessorEn(){
		
		try {
			initialiseSimple();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	public TextProcessorEn(String resourceDir){
		
		this.resourceDir = resourceDir;
		try {
			initialise();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public int getNumberOfWords(ArrayList<String> sent){
		int wordctr = 0;
		for (String s : sent) {
			String[] words = s.split("[\t\n ]");
			for (String w:words) if (!w.trim().equals("")) wordctr ++;
		}

		return wordctr;
	}
	
	public void initialise() throws IOException{

		// only pretrained models are loaded here
		// for training - separate classes are used 
		// TODO: training
		
		String tagsetFileName = resourceDir + "/POStagset.txt";
		readTagsetFromFile(tagsetFileName);

		String resourcePathS = resourceDir + "/en-sent.bin";		
		String resourcePathT = resourceDir + "/en-token.bin";		
		String resourcePathP = resourceDir + "/en-pos-maxent.bin";		
	
	
		// for sentence detecting
		InputStream modelSent = new FileInputStream(resourcePathS);
		SentenceModel modelS = new SentenceModel(modelSent);
				
		sentenceDetector = new SentenceDetectorME(modelS);
		System.out.println("Sentence Detector created");

		// for tokenization		
		InputStream modelTok = new FileInputStream(resourcePathT);
		TokenizerModel modelT = new TokenizerModel(modelTok);
		
		tokenizer = new TokenizerME(modelT);
		System.out.println("Tokeniser created");

		// for POS
		InputStream modelPOS = new FileInputStream(resourcePathP);		
		POSModel modelP = new POSModel(modelPOS);

		tagger = new POSTaggerME(modelP);
		System.out.println("POS tagger created");
		
		
		tokenTagger = new TokenTaggerEn (resourceDir + "/token_tagset.txt");

		
	}
	
	public void initialiseSimple() throws IOException{


		
	}

	
	public ArrayList<String> getSentences (File f){
		ArrayList<String> list = new ArrayList<String>();
		
		try{
			
			Scanner s = new Scanner(f);
			
		    // creates a StanfordCoreNLP object, with POS tagging, lemmatization, NER, parsing, and coreference resolution 
		    Properties props = new Properties();
		    props.put("annotators", "tokenize, ssplit");
		    StanfordCoreNLP pipeline = new StanfordCoreNLP(props);
		    	    

			String text = "";
			while (s.hasNextLine()) {
				
				text += s.nextLine() + "\n";
				
			}
			
		    // create an empty Annotation just with the given text
		    Annotation document = new Annotation(text);
		    

		    pipeline.annotate(document);
		    
		    // these are all the sentences in this document
		    // a CoreMap is essentially a Map that uses class objects as keys and has values with custom types
		    List<CoreMap> sentences = document.get(SentencesAnnotation.class);
		    
		    for(CoreMap sentence: sentences) {

				list.add(sentence.toString());

//				writeVerticalFormatFromCoreMap(text, sentence, out);
		
			}


			
			s.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
		
		return list;
	}
	
	public void processDir (String inDir, String outDir, String specialEnding) {
		
		System.out.println("Input: " + inDir);
		
		FileHandler fh = new FileHandler();
		
		try {
			List<File> list = fh.getFileListing(new File(inDir));

			for (File f : list) {
				
				if (!f.isFile()) continue;
				if (!specialEnding.equals("") 
						&& !f.getName().endsWith(specialEnding)) continue;
					
				String outFile = f.getAbsolutePath().replace(inDir, outDir);
				
				if (new File(outFile).exists()) continue;
				
				String outFile_SEG = f.getAbsolutePath().replace(inDir, outDir.substring(0, outDir.length()-1)+"_SEG/");
				fh.makeParent(outFile, "/");
				fh.makeParent(outFile_SEG, "/");
				
				System.out.println("Processing... " + f.getAbsolutePath());
//				processFileWithOpenNLP(f, new File(outFile), new File(outFile_SEG));
				processFileWithCoreNLP(f, new File(outFile), new File(outFile_SEG));
				
			}
		
		
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
	}
	
	
	
	
	public void processFileWithCoreNLP (File inFile, File outFile, File outFile_SEG) throws IOException {

		if (outFile.exists()) return;
		
		Scanner s = new Scanner(inFile);
		Writer out = new OutputStreamWriter(new FileOutputStream(outFile), ENCODING);
		Writer out_SEG = new OutputStreamWriter(new FileOutputStream(outFile_SEG), ENCODING);
	      
		String text = "";
		while (s.hasNext()) {
			text = s.nextLine() + "\n";		
		

	    // creates a StanfordCoreNLP object, with POS tagging, lemmatization, NER, parsing, and coreference resolution 
	    Properties props = new Properties();
	    props.put("annotators", "tokenize, ssplit, pos, lemma");
	    StanfordCoreNLP pipeline = new StanfordCoreNLP(props);
	    	    
	    // create an empty Annotation just with the given text
	    Annotation document = new Annotation(text);
	    
	    // run all Annotators on this text
	    try{
	    	pipeline.annotate(document);
	    } catch(Exception e){
	    	e.printStackTrace();
	    	return;
	    }
	    
	    // these are all the sentences in this document
	    // a CoreMap is essentially a Map that uses class objects as keys and has values with custom types
	    List<CoreMap> sentences = document.get(SentencesAnnotation.class);
	    
	    for(CoreMap sentence: sentences) {
	      // traversing the words in the current sentence
	      // a CoreLabel is a CoreMap with additional token-specific methods
//	    	System.out.println("S\n"+sentence.toString()+"\n\\S\n");

			writeUnformattedSentence(sentence.toString(), out_SEG);

			writeVerticalFormatFromCoreMap(text, sentence, out);
	    	
			sent_ctr ++;
	
	      // this is the parse tree of the current sentence
//	      Tree tree = sentence.get(TreeAnnotation.class);
	
	      // this is the Stanford dependency graph of the current sentence
//	      SemanticGraph dependencies = sentence.get(CollapsedCCProcessedDependenciesAnnotation.class);
	    }
	    
		}
	
	    // This is the coreference link graph
	    // Each chain stores a set of mentions that link to each other,
	    // along with a method for getting the most representative mention
	    // Both sentence and token offsets start at 1!
//	    Map<Integer, CorefChain> graph = 
//	      document.get(CorefChainAnnotation.class);

		s.close();
		
		out.flush();
		out.close();
		out_SEG.flush();
		out_SEG.close();

	}
	
	
	public String processTextWithCoreNLP (String text) throws IOException {

		String result = "";
		

	    // creates a StanfordCoreNLP object, with POS tagging, lemmatization, NER, parsing, and coreference resolution 
	    Properties props = new Properties();
	    props.put("annotators", "tokenize, ssplit, pos, lemma");
	    StanfordCoreNLP pipeline = new StanfordCoreNLP(props);
	    	    
	    // create an empty Annotation just with the given text
	    Annotation document = new Annotation(text);
	    
	    // run all Annotators on this text
	    try{
	    	pipeline.annotate(document);
	    } catch(Exception e){
	    	e.printStackTrace();
	    	return "";
	    }
	    
	    // these are all the sentences in this document
	    // a CoreMap is essentially a Map that uses class objects as keys and has values with custom types
	    List<CoreMap> sentences = document.get(SentencesAnnotation.class);
	    
	    for(CoreMap sentence: sentences) {
	      // traversing the words in the current sentence
	      // a CoreLabel is a CoreMap with additional token-specific methods
//	    	System.out.println("S\n"+sentence.toString()+"\n\\S\n");

			result += getVerticalFormatFromCoreMap(text,sentence);
	    	
	    }
	    
	    return result;
	}

	public Annotation getAnnotationWithCoreNLP (String text) throws IOException {

		// creates a StanfordCoreNLP object, with POS tagging, lemmatization, NER, parsing, and coreference resolution 
	    Properties props = new Properties();
	    props.put("annotators", "tokenize, ssplit, pos, lemma");
	    StanfordCoreNLP pipeline = new StanfordCoreNLP(props);
	    	    
	    // create an empty Annotation just with the given text
	    Annotation document = new Annotation(text);
	    
	    // run all Annotators on this text
	    try{
	    	pipeline.annotate(document);
	    } catch(Exception e){
	    	e.printStackTrace();
	    	return null;
	    }
	    
	    // these are all the sentences in this document
	    // a CoreMap is essentially a Map that uses class objects as keys and has values with custom types
	    List<CoreMap> sentences = document.get(SentencesAnnotation.class);
	    
	    for(CoreMap sentence: sentences) {
	      // traversing the words in the current sentence
	      // a CoreLabel is a CoreMap with additional token-specific methods
//	    	System.out.println("S\n"+sentence.toString()+"\n\\S\n");

//			result += getVerticalFormatFromCoreMap(text,sentence);
	    	
	    }
	    
	    return document;
	}


	public String[] getTokensWithOpenNLP (String text) throws IOException {
		

		
			String tokens[] = tokenizer.tokenize(text);
			   
			return tokens;		
	}

	
	public void processFileWithOpenNLP (File inFile, File outFile, File outFile_SEG) throws IOException {
		
		Scanner s = new Scanner(inFile);
		Writer out = new OutputStreamWriter(new FileOutputStream(outFile), ENCODING);
		Writer out_SEG = new OutputStreamWriter(new FileOutputStream(outFile_SEG), ENCODING);
	      
		String text = "";
		while (s.hasNext()) {
			text += s.nextLine() + "\n";		
		}
		
		String[] sentences = sentenceDetector.sentDetect(text);							

		
		for (String sentence : sentences) {
		
			String tokens[] = tokenizer.tokenize(sentence);
			   
			if (tokens == null) continue;
			   
			String POStags[] = tagger.tag(tokens);
			
			writeUnformattedSentence(sentence, out_SEG);
			writeVerticalFormatFromArray(tokens, POStags, out);
			
			sent_ctr++;

	   }
		   
		s.close();
		
		out.flush();
		out.close();
		out_SEG.flush();
		out_SEG.close();
		
	}
	
	public void writeUnformattedSentence(String sentence, Writer out_SEG) throws IOException{
		
		out_SEG.write("<S id=\"" + (sent_ctr) + "\">");
		out_SEG.write("\n");
		out_SEG.write(sentence);
		out_SEG.write("\n");
		out_SEG.write("</S>");
		out_SEG.write("\n");
		
	}
	
	// using OpenNLP output
	public void writeVerticalFormatFromArray(String[] tokens, String[] POStags, Writer out) throws IOException{
		
		
		
		out.write("<S id=\"" + (sent_ctr) + "\">");
		out.write("\n");

		for (int i = 0; i < Math.min(tokens.length, POStags.length); i++) {
			out.write(tokens[i] + "\t");
			out.write(getTokenTag(tokens[i])
					+ (i==tokens.length-1? "<S>" : "")
					+ "\t");
			
			// lemma missing
			out.write("" + "\t");
			
			out.write((tagset.containsKey(POStags[i])? tagset.get(POStags[i]) : defaultTag));
			out.write("\n");
		}

		out.write("</S>");
		out.write("\n");
		
	}
	
	// using CoreNLP output
	public void writeVerticalFormatFromCoreMap(CoreMap sentence, Writer out) throws IOException{
		
		String sent = sentence.toString();
		
		out.write("<S id=\"" + (sent_ctr) + "\">");
		out.write("\n");

		List<CoreLabel> list = sentence.get(TokensAnnotation.class);
		
		for (int i = 0; i< list.size(); i++) {
	    	 
			CoreLabel token = list.get(i);
			
	        // this is the text of the token
	        String word = token.get(TextAnnotation.class);
	        // this is the POS tag of the token
	        String pos = token.get(PartOfSpeechAnnotation.class);
	        String lemma = token.get(LemmaAnnotation.class);
	        // this is the NER label of the token
//	        String ne = token.get(NamedEntityTagAnnotation.class);       
	        
	        // wordform 
			out.write(word + "\t");
			
			// tokentag
			out.write(tokenTagger.getTokenTag(word)
					+ "\t");
			
			// delimiter with previous word
			String del = "";
			
			if (i>0) {
				String prevWord = list.get(i-1).get(TextAnnotation.class);
				int b = sent.indexOf(prevWord);
				
				System.out.println(sent);
				System.out.println(prevWord);
				System.out.println(word);
				System.out.println("B="+b);
			
				if (b>-1){
					int e = b + prevWord.length() + sent.substring(b+prevWord.length()).indexOf(word);
					
					System.out.println(sent.substring(b+prevWord.length())+"\nINDEX "+sent.substring(b+prevWord.length()).indexOf(word));
					System.out.println("E="+e);
					
					if (sent.substring(b+prevWord.length()).indexOf(word)>-1) {
						del = sent.substring(b+prevWord.length(),e);
						sent = sent.substring(e);
						System.out.println("del="+del);

					}
				}
			}
			
			del = del.replaceAll("\n", "nn").replaceAll("\t", "tt");
			out.write(del + "\t");
			
			// lemma 
			out.write(lemma + "\t");
			
	
			// pos
//			out.write(pos);
			out.write((tagset.containsKey(pos)? tagset.get(pos) : defaultTag));
			out.write("\n");

	        
	      }
		

		out.write("</S>");
		out.write("\n");
		
		System.exit(0);
		
	}

	// using CoreNLP output
	public void writeVerticalFormatFromCoreMap(String sent, CoreMap sentence, Writer out) throws IOException{
		
		out.write("<S id=\"" + (sent_ctr) + "\">");
		out.write("\n");

		List<CoreLabel> list = sentence.get(TokensAnnotation.class);
		
		for (int i = 0; i< list.size(); i++) {
	    	 
			CoreLabel token = list.get(i);
			
	        // this is the text of the token
	        String word = token.get(TextAnnotation.class);
	        // this is the POS tag of the token
	        String pos = token.get(PartOfSpeechAnnotation.class);
	        String lemma = token.get(LemmaAnnotation.class);
	        // this is the NER label of the token
//	        String ne = token.get(NamedEntityTagAnnotation.class);       
	        
	        // wordform 
			out.write(word + "\t");
			
			// tokentag
			out.write(tokenTagger.getTokenTag(word)
					+ "\t");
			
			// delimiter with previous word
			String del = "";
			
			if (i>0) {
				String prevWord = list.get(i-1).get(TextAnnotation.class);
				int b = sent.indexOf(prevWord);
				
				System.out.println(sent);
				System.out.println(prevWord);
				System.out.println(word);
				System.out.println("B="+b);
			
				if (b>-1){
					int e = b + prevWord.length() + sent.substring(b+prevWord.length()).indexOf(word);
					
					System.out.println(sent.substring(b+prevWord.length())+"\nINDEX "+sent.substring(b+prevWord.length()).indexOf(word));
					System.out.println("E="+e);
					
					if (sent.substring(b+prevWord.length()).indexOf(word)>-1) {
						del = sent.substring(b+prevWord.length(),e);
						sent = sent.substring(e);
						System.out.println("del="+del);

					}
				}
			}
			
			del = del.replaceAll("\n", "nn").replaceAll("\t", "tt");
			out.write(del + "\t");
			
			// lemma 
			out.write(lemma + "\t");
			
	
			// pos
//			out.write(pos);
			out.write((tagset.containsKey(pos)? tagset.get(pos) : defaultTag));
			out.write("\n");

	        
	      }
		

		out.write("</S>");
		out.write("\n");
		
	}

	// using CoreNLP output
	public String getVerticalFormatFromCoreMap(String sent, CoreMap sentence) throws IOException{
		
		String res = "";

		List<CoreLabel> list = sentence.get(TokensAnnotation.class);
		
		for (int i = 0; i< list.size(); i++) {
	    	 
			CoreLabel token = list.get(i);
			
	        // this is the text of the token
	        String word = token.get(TextAnnotation.class);
	        // this is the POS tag of the token
	        String pos = token.get(PartOfSpeechAnnotation.class);
	        String lemma = token.get(LemmaAnnotation.class);
	        // this is the NER label of the token
//	        String ne = token.get(NamedEntityTagAnnotation.class);       
	        
	        // wordform 
			res+= word + "\t";
			
			// tokentag
			res+= "\t";
			
			// delimiter with previous word
			String del = "";
			
			if (i>0) {
				String prevWord = list.get(i-1).get(TextAnnotation.class);
				int b = sent.indexOf(prevWord);
				
				System.out.println(sent);
				System.out.println(prevWord);
				System.out.println(word);
				System.out.println("B="+b);
			
				if (b>-1){
					int e = b + prevWord.length() + sent.substring(b+prevWord.length()).indexOf(word);
					
					System.out.println(sent.substring(b+prevWord.length())+"\nINDEX "+sent.substring(b+prevWord.length()).indexOf(word));
					System.out.println("E="+e);
					
					if (sent.substring(b+prevWord.length()).indexOf(word)>-1) {
						del = sent.substring(b+prevWord.length(),e);
						sent = sent.substring(e);
						System.out.println("del="+del);

					}
				}
			}
			
			del = del.replaceAll("\n", "nn").replaceAll("\t", "tt");
			res+= del + "\t";
			
			// lemma 
			res+= lemma + "\t";
			
	
			// pos
//			out.write(pos);
			res+= (tagset.containsKey(pos)? tagset.get(pos) : defaultTag);
			res+= "\n";

	        
	      }
		

		return res;
		
	}

	
	public String getTokenTag(String token) {
		
		return "TOK_LAT";
	}

	
	  /** Read the contents of the given file. */
	  void readTagsetFromFile(String tagsetFileName) throws IOException {
		  
	    String NL = System.getProperty("line.separator");
	    Scanner scanner = new Scanner(new File(tagsetFileName), ENCODING);
	    

	    if (scanner.equals(null)) return;
	    
	    System.out.println("Scanner created");
	    
	    try {
	      while (scanner.hasNextLine()){
	        String rec = scanner.nextLine();
	        
	        rec = rec.trim();
	        if (rec.length() < 2) continue;
	        if (rec.charAt(0) == '#') 
	        	if (rec.substring(1).trim().equalsIgnoreCase("end")) break;
	        	else continue;
	        String[] entry = rec.split("\t");
	        if (entry.length != 2) continue;
	        
	        if (entry[0].trim().equals("")) {
	        	defaultTag = entry[1];
	        } else {
	        	tagset.put(entry[0].trim(),entry[1].trim());
//		        System.out.println(entry[0] + "\t" + entry[1]);
	        }
	        

	      }
	      	      
	    }
	    finally{
	     scanner.close(); 
	    }
	  }



}
