package bg.bas.dcl.semantic.lexical_db;

import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class BulNet extends HashMap<String, Synset> {
	
	Document doc;
	HashMap<String,ArrayList<String>> index = 
			new HashMap<String,ArrayList<String>>();
	HashMap<String,ArrayList<String>> hyponymIndex = 
			new HashMap<String,ArrayList<String>>();
	
	public BulNet(String xmlFile){
		initialiseFromXML(xmlFile);	 
	
		createIndex();
	
}
	
	public void createIndex(){
		
		for (String id : keySet()) {
			
			Synset s = get(id);
			
			for (Literal l : s.literals){
				ArrayList<String> ind = new ArrayList<String> ();
				if (index.containsKey(l.lemma)) {
					ind = index.get(l.lemma);
				}
				ind.add(id);
				index.put(l.lemma, ind);
			}
			
			
			for (Relation r : s.relations) {
				if (r.type.equalsIgnoreCase("hypernym")) {
					ArrayList<String> ind = new ArrayList<String> ();
					if (hyponymIndex.containsKey(r.ID)) {
						ind = hyponymIndex.get(r.ID);
					}
					ind.add(id);
					hyponymIndex.put(r.ID, ind);


//					System.out.println("SS: "+r.ID + " "+r.type);
				}
			}
			
		}

		
		
	}
	
	public BulNet(String xmlFile,String MSfile){
	
	
			initialiseFromXML(xmlFile);	 
		
			assignMorphosemantic(MSfile);
			createIndex();
			System.out.println("Indexing... " + index.size());
	}
	
	
	public void initialiseFromXML(String xmlFile){
		

		try {

			System.out.println("Reading BulNet from XML...");
			
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			
			doc = dBuilder.parse(new File(xmlFile));
			
			//optional, but recommended
			//read this - http://stackoverflow.com/questions/13786607/normalization-in-dom-parsing-with-java-how-does-it-work
			doc.getDocumentElement().normalize();
			
			NodeList synsetList = doc.getElementsByTagName("SYNSET");
			
			for (int i = 0; i < synsetList.getLength(); i++) {
				 
				Node synsetNode = synsetList.item(i);
				
				if (synsetNode.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element element = (Element) synsetNode; 
				
				addSynsetFromElement(element);

			}

			System.out.println("BulNet read: " + this.size() + " synsets.");
		
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	

	}
	
	
	public void addSynsetFromElement(Element element){
		
		Synset synset = new Synset();
		
		String ID = getSynsetInfo(element, "ID");
		String pos = getSynsetInfo(element, "POS");
		String def = getSynsetInfo(element, "DEF");
		String semclass = getSynsetInfo(element, "SEM_CLASS");
		
		ArrayList<Literal> literals = getSynsetSynonyms(element);
		ArrayList<Relation> relations = getSynsetRelations(element);

		synset.ID = ID;
		synset.pos = pos;
		synset.definition = def;
		synset.semclass = semclass;

		synset.literals = literals;
		synset.relations = relations;
	
		ArrayList<String> us = getSynsetUsages(element);
		synset.usages = us;
		

		put(ID, synset);
		
//		System.out.println(synset.toXML());
		
	}

	
	public String getSynsetInfo(Element element, String what){
		
		
		
		String result = "";
		
		if (element.hasAttribute(what)) {
			result = element.getAttribute(what);
		}
		if (element.hasAttribute(what.toLowerCase())) {
			result = element.getAttribute(what.toLowerCase());
		}
		
		if (!result.equals("")) return result;

		NodeList list = element.getElementsByTagName(what);
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			result = e.getTextContent();
			

		}


		return result;
		
	}

	
	public ArrayList<Literal> getSynsetSynonyms(Element element){
		
		ArrayList<Literal> result = new ArrayList<Literal>();
		
		NodeList list = element.getElementsByTagName("SYNONYM");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			//
			NodeList list1 = e.getElementsByTagName("LITERAL");
			
			for (int j = 0; j < list1.getLength(); j++) {
				 
				Node node1 = list1.item(j);
				
				if (node1.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element e1 = (Element) node1; 
				
				Literal lit = new Literal();
				
				lit.word = e1.getChildNodes().item(0).getTextContent();
				
				lit.lemma = getSynsetInfo(e1, "LEMMA");
				lit.sense = getSynsetInfo(e1, "SENSE");
				lit.status = getSynsetInfo(e1, "STATUS");
				
				NodeList listk = e1.getElementsByTagName("LR");
				
				for (int k = 0; k < listk.getLength(); k++) {
					 
					Node nodek = listk.item(k);
					
					if (nodek.getNodeType() != Node.ELEMENT_NODE) continue;
					
					Element ek = (Element) nodek; 
					
					Relation r = new Relation();
					
					r.ID = ek.getChildNodes().item(0).getTextContent();
					r.type = getSynsetInfo(ek, "TYPE");
					
					lit.relations.add(r);

				}


				
				//
				result.add(lit);

			}


		}


		return result;
		
	}

	public ArrayList<Relation> getSynsetRelations(Element element){
		
		ArrayList<Relation> result = new ArrayList<Relation>();
		
		NodeList list = element.getElementsByTagName("ILR");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			Relation r = new Relation();
			
			r.ID = e.getChildNodes().item(0).getTextContent();
			r.type = getSynsetInfo(e, "TYPE");
			
			result.add(r);

		}
		

		return result;
		
	}
	
	public ArrayList<String> getSynsetUsages(Element element){
		
		ArrayList<String> result = new ArrayList<String>();
		
		NodeList list = element.getElementsByTagName("USAGE");
		
		for (int i = 0; i < list.getLength(); i++) {
			 
			Node node = list.item(i);
			
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;
			
			Element e = (Element) node; 
			
			if (e != null
					&& e.getChildNodes().item(0) != null)
				result.add(e.getChildNodes().item(0).getTextContent());

		}


		return result;
		
	}


	
	public String toXML(){
		
		String result = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		
		result += "<WN>\n";
		
		for (String s : keySet()){
			Synset syn = get(s);
			result += syn.
					toXML() + "\n";
//			System.out.println(syn.toXML());
		}

		result += "</WN>\n";

		return result;

		
	}

	public void saveAsXML(Writer out){
		
		try{
			out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			
			out.write("<WN>\n");
			
			for (String s : keySet()){
				Synset syn = get(s);
				out.write(syn.toXML() + "\n");
//				System.out.println(syn.toXML());
			}
	
			out.write("</WN>\n");

		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	
	public void assignMorphosemantic(String MSfile) {
		 
		try{
			Scanner s = new Scanner(new File(MSfile));
			
			while (s.hasNextLine()) {
				String l = s.nextLine();
				String[] data = l.split("\t");
				
				
			
				if (data.length < 5) continue;
				
				if (!containsKey("eng-30-"+data[4])) continue;
				
				Synset w2 = get("eng-30-"+data[4]);
				
				
				Relation r = new Relation();
				r.ID = "eng-30-"+data[1].trim();
				r.sem = data[2].trim();
				
				w2.relations.add(r);
				put("eng-30-"+data[4], w2);
								
			}
			
			s.close();
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	public ArrayList<Synset> getAllSynsetsContainingLiteral(String literal) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		ArrayList<String> list = index.get(literal);
		
		if (list!=null){
			for (String key : list) {
				result.add(get(key));
			}
		}
		
		return result;
	}
	public ArrayList<Synset> getAllSynsetsContainingLiteral(String literal, boolean verified) {
		
		if (!verified) return getAllSynsetsContainingLiteral(literal);
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		ArrayList<String> list = index.get(literal);
		
		if (list!=null){
			for (String key : list) {
				Synset s = get(key);
				if (s.definition.contains("MT")) continue;
				
				for (Literal l : s.literals) {
					if (l.lemma.equals(literal)) {
						if (!l.status.equals("not verified")) result.add(s);
						break;					
					}
				}
			}
		}
		
		return result;
	}
	
	public ArrayList<Synset> getAllSynsetsFromPos(String pos) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
		
		for (String s : keySet()){
			Synset syn = get(s);
			if (syn.pos.toLowerCase().startsWith(pos.toLowerCase())){
				result.add(syn);
			}
		}
		
		return result;
	}


	public ArrayList<Synset> getAllHyponyms(Synset synset) {
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
				
		ArrayList<String> list = hyponymIndex.get(synset.ID); 
		
		if (list!=null){
			for (String key : list) {
				result.add(get(key));
			}
		}
		
		return result;
	}

	public ArrayList<Synset> getAllHyponymsRecursively(Synset synset) {
		
		
		ArrayList<Synset> result = new ArrayList<Synset>(); 
				
		ArrayList<String> list = hyponymIndex.get(synset.ID); 
		
		if (list!=null){
			for (String key : list) {
				result.add(get(key));
				
				result.addAll(
						getAllHyponymsRecursively( get(key) ));
			}
		}
		
		return result;
	}

}
