package bg.bas.dcl.semantic.lexical_db;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream.GetField;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import edu.mit.jwi.Dictionary;
import edu.mit.jwi.IDictionary;
import edu.mit.jwi.item.IIndexWord;
import edu.mit.jwi.item.ILexFile;
import edu.mit.jwi.item.ISenseKey;
import edu.mit.jwi.item.ISynset;
import edu.mit.jwi.item.IVerbFrame;
import edu.mit.jwi.item.IWord;
import edu.mit.jwi.item.IWordID;
import edu.mit.jwi.item.POS;
import edu.mit.jwi.item.Synset;
import edu.mit.jwi.item.SynsetID;
import edu.mit.jwi.item.VerbFrame;
import edu.stanford.nlp.util.StringUtils;

public class MITJavaWordNetAccess {

    public IDictionary dict;
    
    public MITJavaWordNetAccess(){
        String wordNetDirectory = "/home/ivelina/workspace/TextSimilarity/"
        		+ "external_lib/WordNet-3.0";
        String path = wordNetDirectory + "/dict";
        String dpath = "/home/ivelina/workspace/TextSimilarity/"
        		+ "data/wneng30.xml";
        URL url;
		try {
			url = new URL("file", null, path);

			
		    //construct the Dictionary object and open it
	        dict = new Dictionary(url);
	        dict.open();

		
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}      

 
    }

    
    
    
    public List<IVerbFrame> getVerbFrames(String id){
    	
    	List<IVerbFrame> list = new ArrayList<IVerbFrame>();
    	
    	try{
        SynsetID synid = new SynsetID(Integer.valueOf(
        		id.replace("eng-30-", "").replace("-v","")), POS.VERB);
        
        ISynset syn = dict.getSynset(synid);
        
        List<IWord> words = syn.getWords();

        for (IWord w : words) {
        	List<IVerbFrame> frames = w.getVerbFrames();
        	list.addAll(frames);
        }
    	}catch(Exception e){
    		e.printStackTrace();
    	}

        return list;
    	
    }
    
    public static void main(String[] args) throws IOException{

        //construct URL to WordNet Dictionary directory on the computer
    	
     	
        MITJavaWordNetAccess wnac = new  MITJavaWordNetAccess();
        
        
        String id = "eng-30-02611106-v";
        
         
        System.exit(0);
        
        // key=relation
        HashMap<String, HashMap<String,Integer>> allData = 
        		new HashMap<String, HashMap<String,Integer>>();
        
        
        /**
        MorphosemanticCalculatorEvaluator msce = 
        		new MorphosemanticCalculatorEvaluator(true);
        
        msce.readMorphosemantic();
        
        for (String key : msce.morphosemantic.keySet()){
        	
        	if (!key.contains("-v")) continue;
        	
        	SynsetID synsetV = new SynsetID(Integer.valueOf(key.replace("-v", "")), POS.VERB);
        	String verbLabel = wnac.dict.getSynset(synsetV).getLexicalFile().getName();
        	
        	HashMap<String, ArrayList<String>> allRel = 
        			msce.morphosemantic.get(key);
        	
        	for (String rel : allRel.keySet()) {
        		
        		ArrayList<String> nounIDs = allRel.get(rel);

        		HashMap<String,Integer> verbData = 
        				new HashMap<String,Integer>();
        		
        		if (allData.containsKey(rel)) verbData = allData.get(rel);
        		
        		for (String nounID : nounIDs) {
                	SynsetID synsetN = new SynsetID(
                			Integer.valueOf(nounID.replace("-n", "")), 
                			POS.NOUN);
                	String nounLabel = wnac.dict.getSynset(synsetN).getLexicalFile().getName();

                	String dataKey = verbLabel + "_" +nounLabel;
                	
        			if (verbData.containsKey(dataKey)) {
        				verbData.put(dataKey, verbData.get(dataKey)+1);
        			} else {
        				verbData.put(dataKey, 1);
        			}
        		}
        		
        		allData.put(rel,verbData);
        		
        	}
        	
        
        
        }
        
        
        
        System.out.println("MS relation" 
        		+ "\t" + "Verb label" 
        		+ "\t" + "Noun label" 
        		+ "\t" + "Frequency"
        		+ "\t" + "%"
        		);
       	for (String ms : allData.keySet()) {
       		int total = 0;
      		for (String nl : allData.get(ms).keySet()) {
       			
      			total += allData.get(ms).get(nl);
       			
       		}
       		for (String nl : allData.get(ms).keySet()) {
       			
       	      	System.out.println(ms 
       	      			+ "\t" + nl.replace("_", "\t") 
       	      			+ "\t" + allData.get(ms).get(nl)
       	      			+ "\t" +  allData.get(ms).get(nl)*100.0/total);
       	     
       			
       		}
       		
       		
       	}
  
 

        // look up first sense of the word "dog "
 //       wnac.getInfo("saver", POS.NOUN);
  //      wnac.getInfo("save", POS.VERB);
        
        */
    }    
    
    
    public void getInfo(String w, POS pos){
        IIndexWord idxWord = dict.getIndexWord(w, pos);
        
        for (int i = 0; i < idxWord.getWordIDs().size(); i++){
	        IWordID wordID = idxWord.getWordIDs().get(i);
	        
	        System.out.println("SynSet: " + wordID.getSynsetID());
	        IWord word = dict.getWord(wordID);
	        ISynset synset = word.getSynset();
	        String LexFileName = synset.getLexicalFile().getName();
	        System.out.println("Id = " + wordID);
	        System.out.println(" Lemma = " + word.getLemma());
	        System.out.println(" Gloss = " + word.getSynset().getGloss());      
	        System.out.println("Lexical Name : "+ LexFileName);
	        

        }

    }
    
    
    public String getLexicalName(String synID){
    	
    	if (synID == null || synID.length()==0) return "";
    	
    	String pos = synID.substring(synID.length()-1, synID.length());
    	String id = synID.substring(0, synID.length()-2);
    	
    	id = id.substring(id.lastIndexOf("-") + 1);


    	
//   	System.out.println(id);
    	
   		if(id == null || id.length()==0 || !StringUtils.isNumeric(id)) return "";
   	
   		SynsetID synsetN = new SynsetID(
    			Integer.valueOf(id), 
    			(pos.equals("n")? POS.NOUN : POS.VERB));
    	
    	if (dict.getSynset(synsetN) == null) return null;
    	
 //   	System.err.println(synID + "\t" + id + "\t" + pos);

    	return dict.getSynset(synsetN).getLexicalFile().getName();
    }

}
