package bg.bas.dcl.semantic.lexical_db;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import bg.bas.dcl.entities.Word;
import bg.bas.dcl.entities.WordModel;
import bg.bas.dcl.semantic.lexical_db.BulNet;
import bg.bas.dcl.semantic.lexical_db.Literal;
import bg.bas.dcl.semantic.lexical_db.MITJavaWordNetAccess;
import bg.bas.dcl.semantic.lexical_db.Relation;
import bg.bas.dcl.semantic.lexical_db.Synset;

public class MorphosemanticProcessor {
	
	public BulNet bulnet;
	public HashMap<String,WordModel> models = new HashMap<String,WordModel>();

	public HashMap<String,Word> allVerbs = new HashMap<String,Word>();
	public HashMap<String,Double> probabilities = new HashMap<String,Double>();
	
	Writer out;
	
	MITJavaWordNetAccess wordnetaccess = new MITJavaWordNetAccess();
	
	int ctr = 0;
	
	public MorphosemanticProcessor(BulNet bulnet, 
			String outFile){
		
		this.bulnet = bulnet;
		
		
		
		try{
			out = new OutputStreamWriter(
					new FileOutputStream(outFile),"UTF-8");
			
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		
	}
	
	public BulNet process(String MSfile, 
			String enMSfile,
			String probfile) throws IOException {

		assignEnglishMorphosemantic(enMSfile);

//		getDerivationalModels(MSfile);

		for (WordModel wm : models.values()) {
			String roles = "";
			for (String r : wm.semRoles) roles += r + "; ";
			System.out.println("-"+ 
					(wm.verbEnding.equals("")? "0" : wm.verbEnding) 
					+ "\t-" 
					+ (wm.nounEnding.equals("")? "0" : wm.nounEnding)+"\t"
					+ roles);
		}
		
		getAllVerbs();

		getProbabilities(probfile);

//		extractModelsSemanticRoles();
//		assignMorphosemantic();
	
		out.flush();
		out.close();
		
		return bulnet;
	}
	
	public void assignMorphosemantic() throws IOException {
		
		System.out.println("Processing...");

		for (String id : bulnet.keySet()) {
			Synset syn = bulnet.get(id);
			
			if (!syn.pos.startsWith("n")) continue;
			
			
			boolean confirmed = false;
			
			for (Relation r : syn.relations) {
				
				if (r.type.equals("morphosemantic")){
					confirmed = true;
					
					System.out.println(
							r.sem);
//					System.exit(0);
					confirmed = true;
					break;
				}
			}

			if (!confirmed) 

						
			for (Literal l : syn.literals) {
				
				for (WordModel wm : models.values()){
					
					System.out.println("Model " + wm.nounEnding 
							+ "\t" + wm.verbEnding);

					
					if (l.lemma.endsWith(wm.nounEnding)) {
						
						
						String verb = l.lemma.substring(0,l.lemma.length() - wm.nounEnding.length());
											
						verb += wm.verbEnding;
						
						System.out.println("Processing " + verb + "\t" + l.lemma);

						
						if (allVerbs.containsKey(verb)) {
							
							if (verb.length() - wm.verbEnding.length() < 3){
/**
								out.write("ERROR: " + l.lemma + "," + syn.ID
										+ "\t"
										+ verb + "\n");
								out.flush();
								
								*/
								continue;
							}
		
							
							String[] verbIDs = allVerbs.get(verb)
									.wn.split(";");

							for (String vid : verbIDs) {
								vid = vid.trim();
								if (vid.equals("")) continue;
								
								ctr++;
								
								double total = 0.0;
								for (String sr : wm.semRoles){
									Double pr = probabilities
											.get(sr + "\t" 
													+ wordnetaccess.getLexicalName(id));
									if (pr != null) total += pr;										
											
								
								}

								for (String sr : wm.semRoles){
									Relation msr = 
										new Relation();
									
									msr.ID = vid;
									msr.sem = sr;
									
									
									
									Double pr = probabilities
											.get(sr + "\t" 
													+ wordnetaccess.getLexicalName(id));
									msr.prob = (pr == null? 0.00 : 100.0*pr/total);
									
									if (msr.prob > 0.0) syn.relations.add(msr);
									
									out.write(
											l.lemma + "," + syn.ID
											+ "\t"
											+ verb + "," + vid
											+ "\t"
											+ sr
											+ "\t"
											+ wordnetaccess.getLexicalName(id)
											+ "\t"
											+ (pr == null? 0.00 : 100.0*pr/total)			
											+"\n");
								
									out.flush();
								}
							}
														
						}
						
					}
					
				}
				
			}
			bulnet.put(id, syn);
			
		}
		System.out.println("Finished: " + ctr + " related pairs of verb-noun synsets.");

		
	}
	
	public void assignEnglishMorphosemantic(String MSfile) {
		
		try{
			Scanner s = new Scanner(new File(MSfile));
			
			while (s.hasNextLine()) {
				String[] data = s.nextLine().split("\t");
			
				if (data.length < 5) continue;
				
				if (!bulnet.containsKey("eng-30-"+data[4])) continue;
				
				Synset noun = bulnet.get("eng-30-"+data[4]);
				
//				if (!noun.pos.toLowerCase().startsWith("n")) continue;
				
				
				Relation r = new Relation();
				r.ID = "eng-30-"+data[1].trim();
				r.sem = data[2].trim();
				
				noun.relations.add(r);
				bulnet.put("eng-30-"+data[4], noun);
								
			}
			
			s.close();
			
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}

	
	public void extractModelsSemanticRoles() throws IOException {
		
		System.out.println("Processing...");

		for (String id : bulnet.keySet()) {
			Synset syn = bulnet.get(id);
			
			if (!syn.pos.startsWith("n")) continue;
			
			
			for (Literal l : syn.literals) {
				
				for (int i = 0; i < models.size(); i++) {
					
					WordModel wm = models.get(i);
					
					ArrayList<String> relations = wm.semRoles;
					
					System.out.println("Model " + wm.nounEnding 
							+ "\t" + wm.verbEnding + "\t" + wm.semRoles.size());

					
					if (l.lemma.endsWith(wm.nounEnding)) {
						
						
						String verb = l.lemma.substring(0,l.lemma.length() - wm.nounEnding.length()) + wm.verbEnding;
						
						System.out.println("Processing " + verb + "\t" + l.lemma);

						
						
						if (allVerbs.containsKey(verb)) {
							
							String[] verbIDs = allVerbs.get(verb)
									.wn.split(";");

							for (String vid : verbIDs) {
								vid = vid.trim();
								if (vid.equals("")) continue;
								
								for (Relation r : syn.relations){
								
									System.err.println("Model " + r.ID + "\t" + r.type);

									
									if (r.type.equals("morphosemantic")
											&& (r).ID.equals(vid)) {

										System.err.println("Model1 " + r.ID + "\t" + r.sem);

										
										if (!relations.contains(r.sem)){
											relations.add(r.sem);
										}
									
									}										
								}
								
							}
														
						}
						
					}
					
					wm.semRoles = relations;
					models.put(wm.verbEnding + "\t" + wm.nounEnding, wm);
					
				}
				
			}
			bulnet.put(id, syn);
			
		}
		
		
		for (WordModel wm : models.values()){
			out.write(
				wm.nounEnding
				+ "\t"
				+ wm.verbEnding
				+ "\t" );
			
			for (String rel : wm.semRoles) out.write(rel + ";");
	
			out.write("\n");
		}

		
		System.out.println("Finished.");

		
	}
	
	public void getDerivationalModels(String MSfile) {
		
		System.out.println("Reading derivational models...");

		try{
			Scanner s = new Scanner(new File(MSfile));
			
			while (s.hasNextLine()) {
				String[] data = s.nextLine().split("\t");
			
				if (data.length < 3) continue;
				
				data[0] = data[0].replace("-", "").replace("0", "");
				data[1] = data[1].replace("-", "").replace("0", "");
				
				String key = data[0] + "\t" + data[1];
				
				WordModel model = new WordModel(data[0], data[1]);
				
				if (models.containsKey(key)) model = models.get(key);
				
				for (int i = 2; i < data.length; i++) {
					
					if (!data[i].trim().equals("") 
							&& !model.semRoles.contains(data[i])) {
						model.semRoles.add(data[i]);
					}
					
				}

				models.put(key, model);
				
			}
			
			System.out.println("Models read: " + models.size() + " items.");
			
			s.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}
	

	public void getProbabilities(String file) {
		
		System.out.println("Reading derivational models...");

		try{
			Scanner s = new Scanner(new File(file));
			
			while (s.hasNextLine()) {
				String[] data = s.nextLine().split("\t");
			
				if (data.length < 4) continue;
				
				String key = data[0] + "\t" + data[1];
				
				Double prob = Double.valueOf(data[3]);
				
				probabilities.put(key, prob);
				
			}
			
			System.out.println("Probabilities read: " + probabilities.size() + " items.");
			
			s.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
	}

	public void getAllVerbs() {
		
		System.out.println("Indexing verbs...");


		for (String id : bulnet.keySet()) {
			Synset syn = bulnet.get(id);
			
			if (!syn.pos.startsWith("v")) continue;
			
			for (Literal l : syn.literals) {
				
				Word w = new Word();
			
				if (allVerbs.containsKey(l.word)) {
					w = allVerbs.get(l.word);
				} else {
					
					w.lemma = l.lemma;
					w.pos = "v";
					
				}
				
				w.wn += id + ";";
				
				allVerbs.put(l.word, w);
				
			}
			
			
		}
		System.out.println("Verbs indexed: " + allVerbs.size() + " items.");


		
	}


}
