package bg.bas.dcl.semantic.lexical_db;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import edu.cmu.lti.jawjaw.db.SQL;
import edu.cmu.lti.jawjaw.db.SQLQuery;
import edu.cmu.lti.jawjaw.pobj.POS;
import edu.cmu.lti.jawjaw.pobj.Synset;
import edu.cmu.lti.jawjaw.util.Configuration;
import edu.cmu.lti.lexical_db.NictWordNet;
import edu.cmu.lti.lexical_db.data.Concept;

public class WordNet extends NictWordNet {
	
	   private static ConcurrentMap<String, List<Synset>> cache;

	    static {
	            if (Configuration.getInstance().useCache()) {
	                    cache = new ConcurrentHashMap<String, List<Synset>>(
	                            Configuration.getInstance().getMaxCacheSize());
	            }
	    }


	   /**
     * Find synset record by synset id key
     * @param synset id e.g. 06142412-n
     * @return synset record
     */
    public Synset findSynsetBySynsetId( String synset ) {
            String key = synset;
            if ( Configuration.getInstance().useCache() ) {
                    List<Synset> cachedObj = cache.get(key);
                    if ( cachedObj != null ) return clone(cachedObj).get(0);
            }
           
            Synset s = null;
                   
            PreparedStatement ps = null;
            ResultSet rs = null;
            try {
                    ps = SQL.getInstance().getPreparedStatement( SQLQuery.FIND_SYNSET_BY_SYNSET );
                    synchronized (ps) {
                            ps.setString(1, synset);
                            rs = ps.executeQuery();
                            if ( rs.next() ) {
                                    s = rsToObject(rs);
                            }
                            rs.close();
//                          ps.close();
                            rs = null;
//                          ps = null;
                    }
            } catch ( SQLException e ) {
                    e.printStackTrace();
            } finally {
                    try {
                            if ( rs != null ) rs.close();
//                          if ( ps != null ) ps.close();
                    } catch ( SQLException e ) {
                            e.printStackTrace();
                    }
            }
           
            if (Configuration.getInstance().useCache()) {
//                  synchronized ( cache ) {
                            if (cache.size() >= Configuration.getInstance().getMaxCacheSize()) {
                                    cache.remove( cache.keySet().iterator().next() );
                            }
                            List<Synset> synsets = new ArrayList<Synset>(1);
                            synsets.add(s);
                            if ( synsets != null && s != null) 
                            	cache.put( key, clone(synsets) );
//                  }
            }
            return s;
    }

    
	   /**
  * Find synset record by synset id key
  * @param synset id e.g. 06142412-n
  * @return synset record
  */
    public Concept findConceptBySynset( String id ) {

 				Synset s1 = findSynsetBySynsetId( id );
				
 				if (s1 == null) return null;
 				
				Concept c = new Concept( id, s1.getPos(), s1.getName(), s1.getSrc() );
				return c;
    }

 
    private static List<Synset> clone( List<Synset> synsets ) {
        List<Synset> cloned = new ArrayList<Synset>( synsets.size() );
        for ( Synset synset : synsets ) {
                cloned.add( synset.clone() );
        }
        return synsets;
    
    }

    private static Synset rsToObject( ResultSet rs ) throws SQLException {
        Synset synset = new Synset(
                rs.getString(1),        
                POS.valueOf( rs.getString(2) ),
                rs.getString(3),
                rs.getString(4)
        );
        return synset;
    }



}
