package bg.bas.dcl.semantic.processing;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.TextPosition;

public class PhraseModelSystem {
	
	public ArrayList<NounPhraseModel> models = new ArrayList<NounPhraseModel>();

	public PhraseModelSystem(String file){
		readModelsFromFile(file);
//		System.out.println("MODELS READ "+models.size());
	}
	
	
	
	public void readModelsFromFile(String f){
		
		try{
			
			Scanner s = new Scanner(new File(f));
			
			
			while (s.hasNextLine()){
				
				String l = s.nextLine().trim();
				
				if (l.trim().equals("")) continue;
				
				String[] data = l.split("\t");
				if (data.length < 2) continue;

				String name = data[0];

				
				NounPhraseModel inm = new NounPhraseModel();
				
				inm.name = name;
				
				if (inm.name.startsWith("$")) {
					inm.name = inm.name.substring(1);
					inm.finalrule = false;
				}
				
				inm.modelstring = l;

				String[] comps = data[1].split(" ");
				
				for (String c : comps){
					
					c = c.trim();
					if (c.trim().equals("")) continue;
									
					boolean isHead = false;
					
					if (c.endsWith("!")) {
						isHead = true;
						c = c.substring(0, c.length()-1);
					}

//					System.out.println("COMPONENT "
//							+ c + " "+isHead);
					
					NounPhraseComponent npc = new NounPhraseComponent();
					npc.readComponentFromString(c,3);
					
					if (isHead){
						npc.isHead = isHead;
						
					}
					
//					System.out.println("COMPONENT READ: "
//							+ npc.form+","+npc.pos+npc.gram + " o:"+npc.optional+ " h:"+npc.isHead);
					
					inm.add(npc);
				}
				
				int k = 0;
				
				if (inm.name.startsWith("N")){
					for (k=0;k<models.size(); k++){
						if (models.get(k).size() < inm.size()) break;
					}
				}else {
					k = models.size();
				}
				
				models.add(k,inm);
				
				
			}
//			for (NounPhraseModel m : models ) System.out.println(m.name);

			
//			System.out.println(this.size());
			s.close();
		} catch(Exception e){
			e.printStackTrace();
		}
		
		
	}

	public String getContext(Sentence sent, int start, int len){
		String res = "";
		for (int i=start; i<sent.size() && i-start<=len; i++){
			res += sent.get(i).wordform+" ";
		}
			
		return res.trim();
	}

	public ArrayList<NounPhrase> findForms(Sentence sent) throws IOException {
		
		
		ArrayList<NounPhrase> nounphrases = new ArrayList<NounPhrase>();
		
		try{
		int start = 0;
		
		
		
main:		while (start<sent.size()) {
			
//			System.out.println("CONTEXT: "
//					+ getContext(sent,start,5));
	
			ArrayList<NounPhrase> possibilities = new ArrayList<NounPhrase>();
			int maxlen = 0;
			
			if (!sent.get(start).MWE.equals("")){
//				System.out.println("MWE start");
				
					if (sent.get(start).remainingAttr.containsKey("s")
							&& sent.get(start).remainingAttr.get("s").endsWith("3")) {
						
						NounPhrase np = new NounPhrase();

						for (int j=start; j<sent.size(); j++){
							if (sent.get(j).MWE.equals(sent.get(start).MWE)){
								np.add(new TextPosition(0,j));
							}
						}
						
						if (np.get(0).sentpos>0 
								&& sent.get(np.get(0).sentpos-1).pos.equals("R")){
							np.add(0,new TextPosition(0,np.get(0).sentpos-1));
							np.isPrepositional = true;
						}
						
						np.isMWE = true;
						nounphrases.add(np);
						
						start = np.get(np.size()-1).sentpos+1;
						continue main;
					}
			}
			
			
			
//			for (String key : models.keySet()) {
models:				for (NounPhraseModel m : models){
	
						if (!m.finalrule) continue;
					
//					System.out.println("CHECK MODEL: "+ m.modelname + "/"+nms.size());
					
					NounPhrase np = m.findMatch(sent, 0, start, models);
					
					
					if (np!=null && np.size()>0){
						np.modelstring = m.modelstring;
						np.type = m.name;
						
						if (np.get(0).sentpos>0 
								&& sent.get(np.get(0).sentpos-1).pos.equals("R")){
							np.add(0,new TextPosition(0,np.get(0).sentpos-1));
							np.isPrepositional = true;
						}

//						nounphrases.add(np);
						possibilities.add(np);
						maxlen = Math.max(maxlen,np.size());
						
//						start = np.get(np.size()-1).sentpos+1;
//						continue main;
					} 
					
				}
			
			if (possibilities.size()>0) {
				for (NounPhrase np : possibilities) {
					if (np.size()<maxlen) continue;
					
					nounphrases.add(np);
					start = np.get(np.size()-1).sentpos+1;
					continue main;
				}
			
			} else {

				start++;
				continue;
			}
			
			
		
		}
		}catch(Exception e){
			e.printStackTrace();
			
		}
		return nounphrases;
}

	
}
